/*!
 * truncator v1.2.0
 * https://github.com/ktsn/truncator
 *
 * @license
 * Copyright (c) 2015-2018 katashin
 * Released under the MIT license
 * https://github.com/ktsn/truncator/blob/master/LICENSE
 */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.Truncator = {})));
}(this, (function (exports) { 'use strict';

function extend(target) {
  for (var _len = arguments.length, objs = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    objs[_key - 1] = arguments[_key];
  }

  objs.forEach(function (obj) {
    Object.keys(obj).forEach(function (key) {
      if (obj[key] !== undefined) target[key] = obj[key];
    });
  });
  return target;
}

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};











var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var Dom = function () {
  function Dom(el) {
    classCallCheck(this, Dom);

    this.el = el;
    this.style = window.getComputedStyle(el, '');
  }

  createClass(Dom, [{
    key: 'text',
    set: function set$$1(val) {
      if ('textContent' in this.el) {
        this.el.textContent = val;
      } else if ('innerText' in this.el) {
        this.el.innerText = val;
      } else {
        throw new Error('The browser does not support text insertion for DOM');
      }
    }
  }, {
    key: 'height',
    get: function get$$1() {
      return parseFloat(this.style.getPropertyValue('height')) || 0;
    }
  }, {
    key: 'lineHeight',
    get: function get$$1() {
      var origHTML = this.el.innerHTML;

      this.el.innerHTML = 'X';
      var height = this.height;
      this.el.innerHTML = origHTML;

      return height;
    }
  }]);
  return Dom;
}();

function dom(el) {
  return new Dom(el);
}

var DEFAULT_OPTIONS = {
  ellipsis: '...'
};

var Truncator = function () {
  function Truncator(el, text, boundary, options, truncate) {
    classCallCheck(this, Truncator);

    this.el = el;
    this.text = text;
    this.boundary = boundary;
    this.options = options;
    this.truncate = truncate;

    this.recalc();
  }

  createClass(Truncator, [{
    key: 'recalc',
    value: function recalc() {
      var _this = this;

      execWithUnfixHeight(this.el.el, function () {
        _this.truncate(_this.el, _this.text, _this.boundary, _this.options);
      });
    }
  }, {
    key: 'restoreText',
    value: function restoreText() {
      this.el.text = this.text;
    }
  }]);
  return Truncator;
}();

function truncate(el, text, options) {
  if (options === null || (typeof options === 'undefined' ? 'undefined' : _typeof(options)) !== 'object') {
    throw new Error('options must be an object');
  }

  var domEl = dom(el);
  var opts = normalizeOptions(options);

  if (typeof opts.height === 'number') {
    return new Truncator(domEl, text, opts.height, opts, truncateByHeight);
  }

  if (typeof opts.line === 'number') {
    return new Truncator(domEl, text, Math.floor(opts.line), opts, truncateByLine);
  }

  if (typeof opts.count === 'number') {
    return new Truncator(domEl, text, Math.floor(opts.count), opts, truncateByCount);
  }

  throw new Error('options must have height, line or count as number');
}

function normalizeOptions(options) {
  var opts = extend({}, DEFAULT_OPTIONS, options);

  if (opts.ellipsis === null) opts.ellipsis = '';

  return opts;
}

// set the element height to auto
// and unlock constraints of min-, max-height during given function is executed
function execWithUnfixHeight(el, fn) {
  var _el$style = el.style,
      height = _el$style.height,
      maxHeight = _el$style.maxHeight,
      minHeight = _el$style.minHeight;


  try {
    el.style.height = 'auto';
    el.style.maxHeight = 'none';
    el.style.minHeight = '0';

    fn();
  } finally {
    // ensure the styles are restored
    el.style.height = height;
    el.style.maxHeight = maxHeight;
    el.style.minHeight = minHeight;
  }
}

function truncateByLine(el, text, line, options) {
  truncateByHeight(el, text, el.lineHeight * line, options);
}

function truncateByHeight(el, text, height, options) {
  el.text = text;

  if (el.height <= height) {
    return;
  }

  truncateImpl(el, text, height, options, 0, text.length);
}

function truncateByCount(el, text, count, options) {
  if (text.length <= count) {
    el.text = text;
    return;
  }

  el.text = text.substring(0, count) + options.ellipsis;
}

function truncateImpl(el, text, maxHeight, options, left, right) {
  var center = Math.floor((left + right) / 2);
  var truncated = text.substring(0, center) + options.ellipsis;
  el.text = truncated;

  if (left >= right - 1) {
    return;
  }

  var height = el.height;

  if (height > maxHeight) {
    truncateImpl(el, text, maxHeight, options, left, center);
  } else {
    // left index should always be included in search space
    // because it might be boundary point of truncation
    truncateImpl(el, text, maxHeight, options, center, right);
  }
}

exports.truncate = truncate;

Object.defineProperty(exports, '__esModule', { value: true });

})));
